/*----------------------------------------------------------------------------
 *
 * File:
 * eas_host_debug.h
 *
 * Contents and purpose:
 * This header defines the host wrapper functions for simulating
 * error conditions.
 *
 * Copyright (C) 2008 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

// sentinel
#ifndef _EAS_HOST_DEBUG_H
#define _EAS_HOST_DEBUG_H

#include "eas_types.h"

typedef enum
{
    eInitError,
    eShutdownError,
    eMallocError,
    eOpenError,
    eReadError,
    ePosError,
    eSeekError,
    eLengthError,
    eDupError,
    eCloseError,
    eNumErrorConditions,
} E_ERROR_CONDITIONS;

#endif

